@@ -20,6 +20,7 @@ import os
 import sys
 import optparse
 from os import path
+from six import binary_type
 
 from sphinx.util.osutil import walk
 from sphinx import __display_version__
@@ -369,6 +370,15 @@ Note: By default this script will not overwrite already created files.""")
             mastertoctree = text,
             language = 'en',
         )
+        if isinstance(opts.header, binary_type):
+            d['project'] = d['project'].decode('utf-8')
+        if isinstance(opts.author, binary_type):
+            d['author'] = d['author'].decode('utf-8')
+        if isinstance(opts.version, binary_type):
+            d['version'] = d['version'].decode('utf-8')
+        if isinstance(opts.release, binary_type):
+            d['release'] = d['release'].decode('utf-8')
+
         if not opts.dryrun:
             qs.generate(d, silent=True, overwrite=opts.force)
     elif not opts.notoc:
